// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

window.ngui(() => {
  return {
    props: ['slug'],
    emits: ['submit', 'cancel'],
    beforeCreate() {
      components.setParent(this)
    },

    // Listen on mounted event...
    mounted() {
      // logic      
      if (this.slug) {
        this.ctx.processManager.fetchManager(this.slug)
          .then((manager) => {
            this.form = manager
            this.form.options.tlsKey = this.form.options.tlsKey || ''
            this.form.options.tlsCert = this.form.options.tlsCert || ''
            this.form.options.tlsCaCert = this.form.options.tlsCaCert || ''
            const endpoint = new URL(this.form.options.url)
            this.endpoint.hostname = endpoint.hostname
            this.endpoint.port = endpoint.port
            this.endpoint.protocol = endpoint.protocol
          })
      } else {
        this.form.service = this.ctx.uuidv4()
        this.form.options.slug = this.form.service
      }
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        timeoutCheck: 0,
        loadingEndpointCheck: false,
        invalidEndpoint: false,
        dialog: {
          createAccessToken: false,
        },
        form: {
          service: '',
          options: {
            slug: '',
            url: '',
            type: 'process-manager',
            redirect: true,
            name: '',
            description: '',
            authenticator: 'standard',
            headerModifier: {
              request: {
                'X-Access-Token': ''
              },
            },
            tlsKey: '',
            tlsCert: '',
            tlsCaCert: '',
          }
        },
        endpoint: {
          protocol: 'http:',
          hostname: '',
          port: 9090,
        },
        protocolOptions: [
          {
            label: 'unencrypted',
            value: 'http:',
          },
          {
            label: 'encrypted',
            value: 'https:',
          },
        ],
      }
    },

    // Define methods for the ui...
    methods: {
      submit() {
        this.form.options.url = this.getEndpoint()
        if (!this.slug) {
          this.ctx.processManager.postManager(this.form.options)
            .then((res) => {
              this.$emit('submit', res, this.form)
            })    
        } else {
          this.ctx.processManager.putManager(this.form.options.slug, this.form.options)
            .then((res) => {
              this.$emit('submit', res, this.form)
            })      
        }
      },
      cancel() {
        this.$emit('cancel')
      },
      getEndpoint(suffix = '') {
        return `${this.endpoint.protocol}//${this.endpoint.hostname}:${this.endpoint.port}${suffix}`
      },
      checkEndpoint() {
        clearTimeout(this.timeoutCheck)
        this.loadingEndpointCheck = true
        this.invalidEndpoint = true
        try {
          new URL(this.getEndpoint())
          this.timeoutCheck = setTimeout(() => {
            this.ctx.httpClient.get(this.getEndpoint()).then(() => {
              this.loadingEndpointCheck = false
              this.invalidEndpoint = false
            })
            .catch(() => {
              this.loadingEndpointCheck = false
              this.invalidEndpoint = true
            })
          }, 1000)  
        } catch (e) {
          // nothing to do
        }

      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.form().append(() => [
          components.caption('general'),
          components.input('ID', 'text', this.form.options.slug, (val) => this.form.options.slug = val).attrs({ readonly: true }),
          components.input('name', 'text', this.form.options.name, (val) => this.form.options.name = val).attrs({ required: true, autofocus: true }),
          components.input('description', 'text', this.form.options.description, (val) => this.form.options.description = val),
          components.separator(),
          components.caption('URL'),
          components.input('protocol', 'select', this.endpoint.protocol, (val) => {
            this.endpoint.protocol = val
            if (val === 'https:' && this.endpoint.port === 9090) {
               this.endpoint.port = 9099
            } else if (val === 'http:' && this.endpoint.port === 9099) {
               this.endpoint.port = 9090
            }
          }).attrs({ options: this.protocolOptions }),
          components.input('hostname', 'text', this.endpoint.hostname, (val) => {
            this.endpoint.hostname = val
            this.checkEndpoint()
          }).attrs({ required: true }),
          components.input('port', 'number', this.endpoint.port, (val) => this.endpoint.port = val).attrs({ required: true }),
          components.separator(),
          components.caption('security'),
          components.renderIf(this.endpoint.hostname !== '', components.button('createAccessToken').attrs({
            fullWidth: false,
            class: 'q-my-md',
            loading: this.loadingEndpointCheck,
            disable: this.invalidEndpoint,
            color: !this.invalidEndpoint && !this.loadingEndpointCheck ? 'positive' : 'primary',
            onClick: () => this.dialog.createAccessToken = !this.dialog.createAccessToken,
          })),
          components.input(
            'accessToken', 
            'password', 
            this.form.options.headerModifier.request['X-Access-Token'], 
            (val) => this.form.options.headerModifier.request['X-Access-Token'] = val,
          ).attrs({ 
            required: true,
            hint: 'accessTokenManagerHelp',
          }),
          components.renderIf(this.endpoint.protocol === 'https:',components.separator()),
          components.renderIf(this.endpoint.protocol === 'https:',components.caption('TLS')),
          components.renderIf(this.endpoint.protocol === 'https:',components.input('tlsKey', 'editor', this.form.options.tlsKey, (val) => this.form.options.tlsKey = val).attrs({ required: false })),
          components.renderIf(this.endpoint.protocol === 'https:',components.input('tlsCert', 'editor', this.form.options.tlsCert, (val) => this.form.options.tlsCert = val).attrs({ required: false })),
          components.renderIf(this.endpoint.protocol === 'https:',components.input('tlsCaCert', 'editor', this.form.options.tlsCaCert, (val) => this.form.options.tlsCaCert = val).attrs({ required: false })), 
        ]).on({
          onSubmit: () => this.submit(),
          onCancel: () => this.cancel(),
        }),
        components.dialog('createAccessToken', this.dialog.createAccessToken, (val) => this.dialog.createAccessToken = val).append(() => [
          components.element('iframe').attrs({
            src: this.getEndpoint('/service/app/#/users/access-tokens?headless=true'),
            style: 'border-style: none;width: 100%; height: 500px',
            allow: 'clipboard-write',
          })
        ])
      ]
    },
  }
})