// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

window.ngui(() => {
  return {

    mixins: [
      processManager.mixin(),
    ],
  
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    created() {
      // logic
      this.ctx.endpoints.toService([this.processManagerUuid], true, false)
      this.processName = this.$attrs.name || this.$route.query.name
      this.instanceUuid = this.$attrs.instance || this.$route.query.instance
      this.processUuid = this.$attrs.process || this.$route.query.process
      this.history = this.$attrs.statusIds || this.$route.query.statusIds || ['']
      this.isUnmounted = false
      this.request.statusId = Array.isArray(this.history) && this.history.length ? this.history[this.history.length - 1] : ''
    },

    // Listen on unmounted event...
    unmounted() {
      // logic
      clearTimeout(this.reconnectId)
      if (this.socket && this.socket.close) {
        this.socket.close(1000)
      }
      this.isUnmounted = true
      this.ctx.httpClient.delete(this.ctx.endpoints.getHttp(`/discovery/${this.processUuid}`))
    },

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        isUnmounted: false,
        reconnectId: 0,
        processName: this.$route.query.name,
        processManagerUuid: this.$route.query.service,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$route.query.process,
        endpoint: '',
        isConnected: true,
        loading: true,
        connecting: true,
        isStatic: true,
        socket: {},
        selects: {},
        response: {},
        columns: [],
        rows: [],
        rowsAsObject: {},
        pagination: components.createDefaultPagination(this.$attrs.pagination || 100),
        maxLength: 5000,
        enabled: true,
        history: this.$route.query.statusIds || [''],
        request: {
          command: 'subscribe',
          statusId: ''
        },
        isinit: false,
        hasModule: false,
        lastStatusId: 0,
      }
    },
  
      
    // Define methods for the ui...
    methods: {
      // functions 
      async init() {
        if (this.isinit) {
          return
        }
        this.isinit = true
        try {
          this.loading = true
          this.processManager = await this.ctx.processManager.fetchManager(this.processManagerUuid)
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
          this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.processSettings = await this.ctx.processManager.fetchProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.processServer = await this.ctx.processManager.fetchServer(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.endpoint = this.ctx.routing.getHttpUrl(this.ctx.processManager.getHost(this.processServer)) 
          this.hasModule = window.processManager.hasModule(this.processSettings.name)

          await this.ctx.httpClient.post(this.ctx.endpoints.getHttp('/discovery'), {
            secure: true,
            url: this.endpoint,
            type: 'process',
            name: this.processUuid,
            slug: this.processUuid,
          })

          this.initSocket()
          this.loading = false
        } catch (e) {
          this.ctx.notify.notifyApiError(e)
          this.loading = false
        }
      },
  
      getRootElement() {
    
        if (!this.$attrs.process) {
          return components.page([31], () => this.init())
        }

        if (this.$attrs.status === undefined || this.$attrs.status !== 0) {
          this.init()
        }
        
        return components.element('NgSection').attrs({ permissions: this.ctx.globals.getPermissions(['processesUpdate']) })
      },

      sendSubscribe() {
        this.loading = true
        this.socket.send(this.request)
      },

      sendUnsubscribe(statusId) {
        this.socket.send({ command: 'unsubscribe', statusId: statusId })
      },

      sendAction(actionId) {
        this.socket.send({ command: 'action', statusId: null, actionId })
        components.notify().positive(actionId)
      },

      sendSubState(statusId) {
        // TODO: We need to close the connection because unsubscribe not worked.
        if (this.socket) {
          this.socket.close(1000)
        }
        this.rowsAsObject = {}
        this.rows = []
        this.columns = []
        this.history.push(statusId)
        if (!this.$attrs.process) {
          components.router().replace({ statusIds: this.history })
        }
        // Assign new statusId to request to send it on connected
        this.request.statusId = statusId
        setTimeout(() => {
          this.initSocket()
        }, 500)
      },

      sendBackSubState() {
        // TODO: We need to close the connection because unsubscribe not worked.
        if (this.socket) {
          this.socket.close(1000)
        }
        this.rowsAsObject = {}
        this.rows = []
        this.columns = []
        this.history.pop()
        if (!this.$attrs.process) {
          components.router().replace({ statusIds: this.history })
        }
        // Assign new statusId to request to send it on connected
        this.request.statusId = this.history[this.history.length - 1]
        this.initSocket()
      },

      getHistoryLabel() {
        return this.history[this.history.length - 1] || 'status'
      },

      getHistoryBackLabel() {
        return (this.history.length >= 2 ? this.history[this.history.length - 2] : '') || 'status'
      },

      getColIdent(index) {
        return `col-${index}`
      },

      handelColumns(event) {
        this.isStatic = event.isStatic
        if (this.isStatic) {
          this.enabled = true
        }
        if (event.columnHeaders) {
          this.columns = event.columnHeaders.map((col) => {
            const ident = this.getColIdent(col.index)
            return {
              i18nLabel: col.title,
              name: ident,
              field: (val) => val && val[ident] ? val[ident].content : '',
              align: 'left',
              sortable: true,
            }
          })
        }
      },

      handleRows(event) {
        if (event.rows) {
          event.rows.forEach((row) => {
            if (!this.rowsAsObject[row.rowId]) {
              this.rowsAsObject[row.rowId] = {}
            }
            
            row.cells.forEach((cell) => {
              this.rowsAsObject[row.rowId][this.getColIdent(cell.columnIndex)] = cell
            })
          })

          if (this.isStatic) {
            this.rows = Object.assign(this.rows, Object.values(this.rowsAsObject));
          } else {
            if (this.rows.length > this.maxLength - 1) {
              this.rows.pop()
            }
            this.rows.unshift(Object.values(JSON.parse(JSON.stringify(this.rowsAsObject)))[0])
            
          }
        }
      },

      eventHandler(event) {
          switch (event.event) {
            case 'init':
              this.loading = false
              this.handelColumns(event)
              break;
            case 'rows':
              if (this.enabled) {
                this.handleRows(event)    
              }
              break;
            default:
              break;
          }
      },
      
      messageHandler(data) {
        if (data.events) {
          data.events.forEach((event) => this.eventHandler(event))
        }
      },
      
      initSocket() {
        if (this.isUnmounted || this.processStatus.status === 0) {
          return
        }
        const { Socket } = this.ctx
        this.connecting = true
        clearTimeout(this.reconnectId)
        this.socket = new Socket()
        this.socket.setAuth(true)
        this.socket.onConnected(() => {
          this.isConnected = true
          this.connecting = false
          this.sendSubscribe()
        })

        this.socket.onClosed((ev) => {
          this.connecting = false
          if (ev.code !== 1000) {
            this.isConnected = false
            this.reconnectId = setTimeout(() => this.initSocket(), 5000)
          }
        })

        this.socket.onError((err) => {
          this.isConnected = false
          this.loading = false
          this.ctx.notify.plugin.error(err.message)
        })

        this.socket.onMessage((data = { type: String, data: Object}) => {
          this.isConnected = true
          try {
            this.messageHandler(data.data)
          } catch (e) {
            console.error(e)
          }
        })
        this.socket.connect(this.ctx.endpoints.getWs(`/service/${this.processUuid}/v1/status`))
      },
      
      getActionMenu(row, cell) {
        return components.menu().attrs({ 
          items: this.getActionMenuItems(cell.options), 
          enableGlobalClick: true,
        }).on({ 
          'onClick:menu': (ident) => {
            switch(cell.format) {
              case 'actions':
                this.sendAction(ident)
                break;
              case 'subStates':
                this.sendSubState(ident)
                break
              default:
                components.notify()
                break
            }
          },
        })
      },
      
      getActionMenuItems(options) {
        const opts = options.map((opt) => ({
          label: opt.text,
          id: opt.id,
        }))
        return opts
      },

      getStatusColor(cell) {
        switch (cell.status) {
          case 'none':
            return 'grey-5'
          case 'ok':
            return 'positive'
          case 'error':
            return 'negative'
          case 'warning':
            return 'orange'
          default:
            return ''
        }
      },

      getStatusIcon(cell) {
        switch (cell.status) {
          case 'none':
            return 'label'
          case 'ok':
            return 'done'
          case 'error':
            return 'notifications'
          case 'warning':
            return 'warning'
          default:
            return ''
        }
      },
      
      getCellComponent(row, cell, type = 'row') {
        let output = ''
        
        switch (cell.format) {
          case 'subStates':
          case 'actions':
            output = this.getActionMenu(row, cell)
            if (type === 'item') {
              output.attrs({
                label: 'show',
                color: this.$attrs.grid ? 'primary' : undefined,
                flat: false,
              })
            }
            break
          case 'bool':
            if (cell.status !== 'none') {
              output = components.badge('', cell.content ? 'done' : 'block', this.getStatusColor(cell))
            } else {
              output = components.badge('', cell.content ? 'done' : 'block', cell.content ? 'positive' : 'grey')
            }
            break;
          case 'numeric':
            if (cell.status !== 'none' && cell.content) {
              output = components.badge(output, this.getStatusIcon(cell), this.getStatusColor(cell))
            } else {
              output = cell.content
            }
            break;
          case 'timestamp':
            if (cell.status !== 'none' && cell.content) {
              output = components.badge(cell.content, this.getStatusIcon(cell), this.getStatusColor(cell))
            } else {
              output = new Date(cell.content).toLocaleString()
            }
            break;
          default:
            if (cell.status !== 'none' && cell.content) {
              output = components.badge(cell.content, this.getStatusIcon(cell), this.getStatusColor(cell))
            } else {
              if (cell.content.includes('\n')) {
                output = components.element('pre').append(() => [ cell.content ])
              } else {
                output = cell.content
              }
            }
            
        }
        return output
      },
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        this.getRootElement().append(() => [
            components.renderIf(!this.$attrs.process,
               components.breadcrumbs(processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid, 'status'))
            ),
            
            components.renderIf(!this.$attrs.process, 
              components.bar('process').createMenu('file', [
                {
                  id: 'player',
                  label: this.enabled ? 'stop' : 'start',
                  icon: this.enabled ? 'stop' : 'play_arrow',
                  disable: this.isStatic,
                  click: () => this.enabled = !this.enabled,
                }
              ]).createMenu('view', [
                {
                  id: 'logs',
                  label: 'logs',
                  icon: 'description',
                  permissions: this.ctx.globals.getPermissions(['changelogRead']),
                  click: () => {
                    components.router().push('/process-manager/instances/processes/logs/index', { 
                      service: this.$route.query.service,
                      instance: this.instanceUuid,
                      process: this.processUuid,
                      processName: this.$route.query.name,
                    })
                  }
                },
  
                {
                  id: 'status',
                  label: 'status',
                  icon: 'checklist',
                  disable: true,
                  active: true,
                  click: () => {
                    components.router().push('/process-manager/instances/processes/status/index', { 
                      service: this.$route.query.service,
                      instance: this.instanceUuid,
                      process: this.processUuid,
                      processName: this.$route.query.name,
                    })
                  }
                },
  
                {
                  id: 'module',
                  label: 'module',
                  icon: 'view_module',
                  disable: !this.hasModule,
                  click: () => {
                    components.router().push(`/modules/${this.processSettings.name}/index`, { 
                      service: this.$route.query.service,
                      instance: this.instanceUuid,
                      process: this.processUuid,
                      processName: this.$route.query.name,
                    })
                  }
                },
            ])
            ),

            components.element(this.$attrs.dense ? 'div' : 'NgCardSection').attrs({ class: 'q-gutter-md' }).append(() => [
              components.renderIf(!this.isConnected, components.bannerNegative(this.ctx.i18n.i18n.t('notConnected'))),
              components.renderIf(this.processStatus.status === 0 || this.$attrs.status === 0, components.bannerWarn(this.ctx.i18n.i18n.t('enableTheProcess'))),
              components.renderIf(this.processStatus.status !== 0 && this.$attrs.status !== 0, components.table(this.lastStatusId, this.columns, this.rows)
                .pagination(this.pagination, (val) => this.pagination = val)
                .attrs({
                  maxHeight: this.$attrs.maxHeight || '80vh',
                  clickable: false,
                  flat: !!this.$attrs.flat,
                  grid: !!this.$attrs.grid,
                  loading: (this.loading || this.connecting) && this.processStatus.status !== 0,
                })
                .append('top-left', () => {
                  const render = [
                    components.renderIf(
                      this.history.length >= 2,
                      components.button(this.getHistoryBackLabel(), 'chevron_left').on({
                        onClick: () => {
                          this.sendBackSubState()
                        }
                      })
                    )
                  ]
                  
                  if (!this.isStatic && !this.$attrs.process) {
                    render.push(components.button('start', 'play_arrow').attrs({
                      disable: this.enabled,
                      color: 'positive',
                    }).on({
                      onClick: () => this.enabled = !this.enabled,
                    }),
                    components.button('stop', 'stop').attrs({
                      color: 'negative',
                      disable: !this.enabled,
                    }).on({
                      onClick: () => this.enabled = !this.enabled,
                    }))
                  }
                  
                  return [
                    components.flex().append(() => render)
                  ]
                })
                .append('body-cell', (props) => {
                  const item = props.row[props.col.name] !== undefined ? props.row[props.col.name] : { content: '' }
                  return [
                    components.td(props).attrs({
                      style: 'min-width: 150px; max-width: 850px;',
                      class: 'text-overflow',
                    }).append(() => [
                      this.getCellComponent(props.row, item),
                    ])
                  ]
                }).append('item', (props) => [
                  components.tdItem(components.card().attrs({ flat: false }).append(() => [
                    components.list().append(() => [
                      ...this.columns.map((col) => {
                        const item = props.row[col.name] !== undefined ? props.row[col.name] : { content: '' }
                        return components.item(col.i18nLabel).append('subtitle', () => [
                          this.getCellComponent(props.row, item, 'item'),
                        ])
                      })
                    ])
                  ]))
                ])),
            ]),     
          ]),
      ]
    },
  }
})